@@ -4,7 +4,7 @@
 
 ;; Author: Oleh Krehel <ohwoeowho@gmail.com>
 ;; URL: https://github.com/abo-abo/swiper
-;; Version: 0.2.0
+;; Version: 0.2.3
 ;; Package-Requires: ((emacs "24.1"))
 ;; Keywords: matching
 
@@ -36,6 +36,8 @@
 ;; re-building it into a regex.
 ;; So "for example" is transformed into "\\(for\\).*\\(example\\)".
 
+(require 'cl-lib)
+
 ;;; Code:
 ;;* Customization
 (defgroup ivy nil
@@ -59,12 +61,18 @@ Set this to nil if you don't want the count."
   "Whether to wrap around after the first and last candidate."
   :type 'boolean)
 
+(defcustom ivy-on-del-error-function 'minibuffer-keyboard-quit
+  "The handler for when `ivy-backward-delete-char' throws.
+This is usually meant as a quick exit out of the minibuffer."
+  :type 'function)
+
 ;;* User Visible
 ;;** Keymap
 (require 'delsel)
 (defvar ivy-minibuffer-map
   (let ((map (make-sparse-keymap)))
     (define-key map (kbd "C-m") 'ivy-done)
+    (define-key map (kbd "C-j") 'ivy-alt-done)
     (define-key map (kbd "C-n") 'ivy-next-line)
     (define-key map (kbd "C-p") 'ivy-previous-line)
     (define-key map (kbd "C-s") 'ivy-next-line-or-history)
@@ -85,16 +93,45 @@ Set this to nil if you don't want the count."
 Maximum length of the history list is determined by the value
 of `history-length', which see.")
 
+(defvar ivy-require-match t
+  "Store require-match. See `completing-read'.")
+
+(defvar ivy--directory nil
+  "Current directory when completing file names.")
+
 ;;** Commands
 (defun ivy-done ()
   "Exit the minibuffer with the selected candidate."
   (interactive)
   (delete-minibuffer-contents)
-  (unless (zerop ivy--length)
-    (insert ivy--current)
+  (if (zerop ivy--length)
+      (when (memq ivy-require-match '(nil confirm confirm-after-completion))
+        (insert ivy-text)
+        (setq ivy-exit 'done))
+    (if ivy--directory
+        (insert (expand-file-name ivy--current ivy--directory))
+      (insert ivy--current))
     (setq ivy-exit 'done))
   (exit-minibuffer))
 
+(defun ivy-alt-done ()
+  "Exit the minibuffer with the selected candidate."
+  (interactive)
+  (if (and ivy--directory
+           (file-directory-p
+            (expand-file-name ivy--current ivy--directory)))
+      (progn
+        (delete-minibuffer-contents)
+        (setq ivy--directory
+              (expand-file-name ivy--current ivy--directory))
+        (setq ivy--old-cands nil)
+        (setq ivy--all-candidates
+              (let ((default-directory ivy--directory))
+                (all-completions "" 'read-file-name-internal)))
+
+        (ivy--exhibit))
+    (ivy-done)))
+
 (defun ivy-beginning-of-buffer ()
   "Select the first completion candidate."
   (interactive)
@@ -105,43 +142,41 @@ of `history-length', which see.")
   (interactive)
   (setq ivy--index (1- ivy--length)))
 
-(defun ivy-next-line ()
-  "Select the next completion candidate."
-  (interactive)
-  (if (>= ivy--index (1- ivy--length))
-      (when ivy-wrap
-        (ivy-beginning-of-buffer))
-    (cl-incf ivy--index)))
+(defun ivy-next-line (&optional arg)
+  "Move cursor vertically down ARG candidates."
+  (interactive "p")
+  (setq arg (or arg 1))
+  (cl-incf ivy--index arg)
+  (when (>= ivy--index (1- ivy--length))
+    (if ivy-wrap
+        (ivy-beginning-of-buffer)
+      (setq ivy--index (1- ivy--length)))))
 
-(defun ivy-next-line-or-history ()
-  "Select the next completion candidate.
+(defun ivy-next-line-or-history (&optional arg)
+  "Move cursor vertically down ARG candidates.
 If the input is empty, select the previous history element instead."
-  (interactive)
+  (interactive "p")
   (when (string= ivy-text "")
     (ivy-previous-history-element 1))
-  (if (>= ivy--index (1- ivy--length))
-      (when ivy-wrap
-        (ivy-beginning-of-buffer))
-    (cl-incf ivy--index)))
+  (ivy-next-line arg))
 
-(defun ivy-previous-line ()
-  "Select the previous completion candidate."
-  (interactive)
-  (if (zerop ivy--index)
-      (when ivy-wrap
-        (ivy-end-of-buffer))
-    (cl-decf ivy--index)))
+(defun ivy-previous-line (&optional arg)
+  "Move cursor vertically up ARG candidates."
+  (interactive "p")
+  (setq arg (or arg 1))
+  (cl-decf ivy--index arg)
+  (when (< ivy--index 0)
+    (if ivy-wrap
+        (ivy-end-of-buffer)
+      (setq ivy--index 0))))
 
-(defun ivy-previous-line-or-history ()
-  "Select the previous completion candidate.
+(defun ivy-previous-line-or-history (arg)
+  "Move cursor vertically up ARG candidates.
 If the input is empty, select the previous history element instead."
-  (interactive)
+  (interactive "p")
   (when (string= ivy-text "")
     (ivy-previous-history-element 1))
-  (if (zerop ivy--index)
-      (when ivy-wrap
-        (ivy-end-of-buffer))
-    (cl-decf ivy--index)))
+  (ivy-previous-line arg))
 
 (defun ivy-previous-history-element (arg)
   "Forward to `previous-history-element' with ARG."
@@ -157,32 +192,54 @@ If the input is empty, select the previous history element instead."
 
 (defun ivy-backward-delete-char ()
   "Forward to `backward-delete-char'.
-On error (read-only), quit without selecting."
+On error (read-only), call `ivy-on-del-error-function'."
   (interactive)
-  (condition-case nil
-      (backward-delete-char 1)
-    (error
-     (minibuffer-keyboard-quit))))
+  (if (and ivy--directory (= (minibuffer-prompt-end) (point)))
+      (progn
+        (setq ivy--old-cands nil)
+        (setq ivy--all-candidates
+              (let ((default-directory (setq ivy--directory
+                                             (file-name-directory
+                                              (directory-file-name ivy--directory)))))
+                (all-completions "" 'read-file-name-internal)))
+        (ivy--exhibit))
+    (condition-case nil
+        (backward-delete-char 1)
+      (error
+       (when ivy-on-del-error-function
+         (funcall ivy-on-del-error-function))))))
 
 ;;** Entry Point
 (defun ivy-read (prompt collection
-                 &optional initial-input keymap preselect update-fn)
+                 &optional predicate initial-input keymap preselect update-fn)
   "Read a string in the minibuffer, with completion.
 
 PROMPT is a string to prompt with; normally it ends in a colon
 and a space.  When PROMPT contains %d, it will be updated with
 the current number of matching candidates.
+See also `ivy-count-format'.
 
 COLLECTION is a list of strings.
 
 If INITIAL-INPUT is non-nil, insert it in the minibuffer initially.
 
-UPDATE-FN is called each time the current candidate(s) is changed.
+KEYMAP is composed together with `ivy-minibuffer-map'.
 
 If PRESELECT is non-nil select the corresponding candidate out of
 the ones that match INITIAL-INPUT.
 
-KEYMAP is composed together with `ivy-minibuffer-map'."
+UPDATE-FN is called each time the current candidate(s) is changed."
+  (setq ivy--directory nil)
+  (cond ((or (functionp collection)
+             (vectorp collection))
+         (when (eq collection 'read-file-name-internal)
+           (setq ivy--directory default-directory)
+           (setq initial-input nil))
+         (setq collection (all-completions "" collection predicate)))
+        ((hash-table-p collection)
+         (error "Hash table as a collection unsupported"))
+        ((listp (car collection))
+         (setq collection (all-completions "" collection predicate))))
   (cl-case (length collection)
     (0 nil)
     (1 (car collection))
@@ -204,6 +261,8 @@ KEYMAP is composed together with `ivy-minibuffer-map'."
                   prompt)
                  ((string-match "%.*d" ivy-count-format)
                   (concat ivy-count-format prompt))
+                 (ivy--directory
+                  prompt)
                  (t
                   nil)))
      (setq ivy--action nil)
@@ -226,6 +285,43 @@ KEYMAP is composed together with `ivy-minibuffer-map'."
        (when ivy--action
          (funcall ivy--action))))))
 
+(defun ivy-completing-read (prompt collection
+                            &optional predicate require-match initial-input
+                              _history def _inherit-input-method)
+  "Read a string in the minibuffer, with completion.
+
+This is an interface that conforms to `completing-read', so that
+it can be used for `completing-read-function'.
+
+PROMPT is a string to prompt with; normally it ends in a colon and a space.
+COLLECTION can be a list of strings, an alist, an obarray or a hash table.
+PREDICATE limits completion to a subset of COLLECTION.
+
+REQUIRE-MATCH is stored into `ivy-require-match'. See `completing-read'.
+INITIAL-INPUT is a string that can be inserted into the minibuffer initially.
+_HISTORY is ignored for now.
+DEF is the default value.
+_INHERIT-INPUT-METHOD is ignored for now.
+
+The history, defaults and input-method arguments are ignored for now."
+  (when (listp def)
+    (setq def (car def)))
+  (setq ivy-require-match require-match)
+  (ivy-read prompt collection predicate initial-input nil def))
+
+;;;###autoload
+(define-minor-mode ivy-mode
+    "Toggle Ivy mode on or off.
+With ARG, turn Ivy mode on if arg is positive, off otherwise.
+Turning on Ivy mode will set `completing-read-function' to
+`ivy-completing-read'."
+  :group 'ivy
+  :global t
+  :lighter " ivy"
+  (if ivy-mode
+      (setq completing-read-function 'ivy-completing-read)
+    (setq completing-read-function 'completing-read-default)))
+
 (defvar ivy--action nil
   "Store a function to call at the end of `ivy--read'.")
 
@@ -331,7 +427,11 @@ When non-nil, it should contain one %d.")
   "Update the prompt according to `ivy--prompt'."
   (when ivy--prompt
     (let ((inhibit-read-only t)
-          (n-str (format ivy--prompt ivy--length)))
+          (n-str
+           (format
+            (if ivy--directory
+                (concat ivy--prompt (abbreviate-file-name ivy--directory))
+              ivy--prompt) ivy--length)))
       (save-excursion
         (goto-char (point-min))
         (delete-region (point-min) (minibuffer-prompt-end))
@@ -384,7 +484,6 @@ CANDIDATES is a list of strings."
          (cands (if (and (equal re ivy--old-re)
                          ivy--old-cands)
                     ivy--old-cands
-                  (setq ivy--old-re re)
                   (ignore-errors
                     (cl-remove-if-not
                      (lambda (x) (string-match re x))
@@ -392,12 +491,15 @@ CANDIDATES is a list of strings."
          (tail (nthcdr ivy--index ivy--old-cands))
          (ww (window-width))
          idx)
-    (setq ivy--length (length cands))
     (when (and tail ivy--old-cands)
-      (while (and tail
-                  (null (setq idx (cl-position (pop tail) cands
-                                               :test #'equal)))))
-      (setq ivy--index (or idx 0)))
+      (unless (and (not (equal re ivy--old-re))
+                   (setq ivy--index (cl-position re cands :test 'equal)))
+        (while (and tail (null idx))
+          ;; Compare with eq to handle equal duplicates in cands
+          (setq idx (cl-position (pop tail) cands)))
+        (setq ivy--index (or idx 0))))
+    (setq ivy--old-re re)
+    (setq ivy--length (length cands))
     (setq ivy--old-cands cands)
     (when (>= ivy--index ivy--length)
       (setq ivy--index (max (1- ivy--length) 0)))
